@@ -20,6 +20,7 @@ import fnmatch
 import json
 import yaml
 from urlparse import urljoin
+import unwrapper
 
 _http = None
 _cache = None
@@ -163,6 +164,7 @@ class Loader(object):
         self._things = {}
         self._schemes = {}
         self._locals = []
+        self._unwrapper = unwrapper.Unwrapper()
 
     def add_scheme(self, prefix, func):
         self._schemes[prefix] = func
@@ -272,6 +274,11 @@ class Loader(object):
             # XXX Allow for more content-types
 
         if url.startswith('http'):
+            # Now use the unwrapper in case it was an indirection URL.
+            res = self._unwrapper.unwrap(url)
+            if 'final' in res:
+                url = res['final']
+
             response, body = _get_http().request(url)
             if response['status'] in ['200', '304']:
                 return response, StringIO(body)
@@ -722,6 +729,13 @@ class NotInMap(Exception):
         self.name = name
         self.map = map
 
+class BadMap(Exception):
+    """When a map has nonsensical specificaion."""
+    def __init__(self, msg, orig_args, orig_kwargs={}):
+        super(BadMap, self).__init__(msg + '\n' + repr(orig_args))
+        self.orig_args = orig_args
+        self.orig_kwargs = orig_kwargs
+
 class GridMap(MapBase):
     def __init__(self, source_box, cell_box, names):
         """Create a grid map with this size and names.
@@ -744,9 +758,12 @@ class GridMap(MapBase):
             im_wd = right - self.im_left
             im_ht = bottom - self.im_top
         self.nx, self.ny = im_wd // self.cell_wd, im_ht // self.cell_ht
-
         self.names = names
 
+        if im_wd < self.cell_wd or  im_ht < self.cell_ht:
+            raise BadMap('Image must be at least as wide and tall as one cell ({0}×{1} image, {2}×{3} cell)'.format(im_wd, im_ht, self.cell_wd, self.cell_ht),
+            (source_box, cell_box, names))
+
     def get_css_background_dimens(self):
         return self.nx * self.cell_wd, self.ny * self.cell_ht
 
