@@ -13,6 +13,7 @@ import unittest
 from mock import Mock, patch
 
 from texturepacker import *
+import texturepacker.unwrapper
 
 from datetime import datetime, timedelta
 from zipfile import ZipFile, ZipInfo, ZIP_DEFLATED
@@ -75,14 +76,14 @@ class TestCase(unittest.TestCase):
 
 
 class TextResourceTests(TestCase):
-	def setUp(self):
-		pass
+    def setUp(self):
+    	pass
 
-	def test_pack_txt(self):
-	    doc = TextResource('pack.txt', u'Test pack\nBy Fréd the Deäd')
-	    self.assertEqual('pack.txt', doc.name)
-	    self.assertEqual('Test pack\nBy Fréd the Deäd', doc.get_bytes())
-	    # This test relies on the encoding of the source file being UTF-8!!
+    def test_pack_txt(self):
+        doc = TextResource('pack.txt', u'Test pack\nBy Fréd the Deäd')
+        self.assertEqual('pack.txt', doc.name)
+        self.assertEqual('Test pack\nBy Fréd the Deäd', doc.get_bytes())
+        # This test relies on the encoding of the source file being UTF-8!!
 
 
 class SourcePackTests(TestCase):
@@ -116,7 +117,6 @@ class SourcePackTests(TestCase):
         pack = SourcePack(file_path, Atlas())
         self.check_pack_is_sign_pack(pack)
 
-
     def test_pack_from_directory(self):
         pack = self.create_sign_directory()
 
@@ -236,6 +236,7 @@ class GridMapTests(TestCase):
         self.assertEqual("background-size: 64px 64px; width: 32px; height: 32px; background-position: 0 -224px;", map2.get_css('yankee', 32))
         self.assertEqual("background-size: 64px 64px; width: 32px; height: 32px; background-position: -32px -224px;", map2.get_css('zulu', 32))
 
+
 class CompositeMapTests(TestCase):
     def test_two_grids(self):
         names1 = [
@@ -299,7 +300,6 @@ class AtlasTests(TestCase):
         self.assertEqual((48, 16, 64, 32), m.get_box('s'))
 
 
-
 class CompositeResourceTests(TestCase):
     def test_change_one(self):
         # Create a pack with 2 resources in it.
@@ -321,6 +321,15 @@ class CompositeResourceTests(TestCase):
         self.assert_PNGs_match(self.get_data('a_b_replace.png'), bytes)
 
 
+# Create a fake URL unwrapper.
+class StubUnwrapper(object):
+    def unwrap(self, url, until=None):
+      return {'final': url}
+unwrapper_class_stub = Mock()
+unwrapper_class_stub.return_value = StubUnwrapper()
+
+
+@patch.object(texturepacker.unwrapper, 'Unwrapper', unwrapper_class_stub)
 class ExternalResourceTests(TestCase):
     def setUp(self):
         super(ExternalResourceTests, self).setUp()
@@ -422,7 +431,6 @@ class ExternalResourceTests(TestCase):
         self.assertTrue(mock_open.call_args[0][1].startswith('r'))
 
 
-
 class ResolveUrlTests(unittest.TestCase):
     # I ended up creating my own generic URL resolver, because
     # the standard library’s urljoin seems reluctant to tackle
@@ -563,6 +571,7 @@ class MixerTests(TestCase):
         pack2 = Mixer().get_pack('file://' + os.path.abspath(file_path))
         self.assert_same_packs(pack1, pack2)
 
+    @patch.object(texturepacker.unwrapper, 'Unwrapper', unwrapper_class_stub)
     @patch('httplib2.Http.request')
     def test_get_pack_from_http(self, mock_meth):
         # Arrange that downloading any URL returns our pack.
@@ -578,6 +587,7 @@ class MixerTests(TestCase):
         self.assert_same_packs(pack1, pack2)
 
     # Identical to the abover except passing the URL as a string not dict.
+    @patch.object(texturepacker.unwrapper, 'Unwrapper', unwrapper_class_stub)
     @patch('httplib2.Http.request')
     def test_get_pack_from_naked_http(self, mock_meth):
         # Arrange that downloading any URL returns our pack.
@@ -597,6 +607,37 @@ class MixerTests(TestCase):
         self.assertEqual('http://example.org/frog.zip', mock_meth.call_args[0][0])
         self.assert_same_packs(pack1, pack2)
 
+
+    # Identical to the above except requires unwrapper to work.
+    @patch.object(texturepacker.unwrapper, 'Unwrapper')
+    @patch('httplib2.Http.request')
+    def test_get_pack_using_unwrapper(self, request_mock, unwrapper_class_mock):
+        # Arrange that downloading any URL returns our pack.
+        unwrapper_mock = Mock()
+        unwrapper_mock.unwrap.return_value = {
+            'download': 'http://example.org/foo.bart',
+            'final': 'http://example.org/toad.zip',
+        }
+        unwrapper_class_mock.return_value  = unwrapper_mock
+
+        pack1, data1 = self.sample_pack_and_bytes()
+        request_mock.return_value = ({
+            'status': '200',
+            'content-type': 'application/zip',
+            'content-length': str(len(data1)),
+        }, data1)
+
+        pack2 = Mixer().get_pack('http://parasite.ly/12345')
+        # Not laoded yet:
+        self.assertFalse(request_mock.call_args)
+
+        res = pack2.get_resource('a.png')
+        # Has now downloaded the data:
+        unwrapper_class_mock.assert_called_once_with()
+        unwrapper_mock.unwrap.assert_called_once_with('http://parasite.ly/12345')
+        self.assertEqual('http://example.org/toad.zip', request_mock.call_args[0][0])
+        self.assert_same_packs(pack1, pack2)
+
     def test_get_pack_from_relative_file(self):
         pack1 = self.sample_pack()
         file_path = os.path.join(self.test_dir, 'zum.zip')
@@ -1188,7 +1229,6 @@ class MixerTests(TestCase):
         self.assertEqual('ababababk', pack.desc)
         self.check_pack(pack, {'ab.png': 'a_b_replace.png'}, ['b.png'])
 
-
     def test_copy_star_png_from_dir_within_dir(self):
         # Test added to find a bug.
         dir_path = os.path.join(self.test_dir, 'flippy')
@@ -1688,6 +1728,7 @@ class TestMixerGetAtlas(TestCase):
         atlas = mixer.get_atlas('http://example.com/mapmap/mapittymap.tpmaps', 'http://example.com/zuer/')
         self.assertEqual('http://example.com/mapmap/mapittymap.tpmaps', mock_request.call_args[0][0])
 
+
 class TestDepaletizationOfPaletizedImages(TestCase):
     def test_red_green(self):
         rb_map = GridMap((32, 32), (16, 16), ['black', 'grey', 'red', 'pink'])
@@ -1727,5 +1768,6 @@ class TestDepaletizationOfPaletizedImages(TestCase):
 
         self.assert_PNGs_match(self.get_data('redgreen.png'), remix.get_resource('redgreen.png'))
 
+
 if __name__ == '__main__':
-	unittest.main()
+    unittest.main()
